#include <stdio.h>
#include <ctype.h>
#include <proto.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "common.h"
#include "io.h"
#include "tag.h"
#include "error.h"
#include "dlink.h"

/************************************************************************\
*
* Main program module for 'tagorg'.
*
* $Copyright: (C) 1997 by Gran Gustafsson $ 
*
\************************************************************************/


/************************************************************************\
*
* Copy source string and add suffix to it's basename.
*
\************************************************************************/
void AppendSuffix(char *src,char *dest,char *ext)
{
	char *pt;
	
	if((pt=strrchr(src,'/')))
		strcpy(dest,pt);
	else
		strcpy(dest,src);
	if((pt=strrchr(dest,'.')))
		*pt=0;
	strcat(dest,ext);
}

/************************************************************************\
*
* Main program entry.
*
\************************************************************************/
int main(int argc,char **argv)
{
	int c,n,nr=0;
	int com=0;
	char *start="GT_USER",*pref="",*out=NULL,*inc="gttags.h";
	char buf[500];
	sbool chg=FALSE,list=FALSE,gen=FALSE;
	sbool verbose=FALSE;
	DLink tlhd;
	
	opterr=0;
	while((c=getopt(argc,argv,"adfclvs:p:o:e:i:"))>=0)
	{
		switch(c)
		{
			case 'a':
				if(com)
					Error(ERR_FATAL,"Multiple commands");
				com='a';
				break;
			case 'd':
				if(com)
					Error(ERR_FATAL,"Multiple commands");
				com='d';
				break;
			case 'f':
				if(com)
					Error(ERR_FATAL,"Multiple commands");
				com='f';
				break;
			case 'c':
				gen=TRUE;
				break;
			case 'l':
				list=TRUE;
				break;
			case 'v':
				verbose=TRUE;
				break;
			case 's':
				start=optarg;
				break;
			case 'p':
				pref=optarg;
				break;
			case 'o':
				out=optarg;
				break;
			case 'e':
				nr=atoi(optarg);
				break;
			case 'i':
				inc=optarg;
				break;
			default:
				printf("Usage: %s [-adfclv] [-e<ext>] [-s<start>] [-p<prefix>] [-i<inc>] [-o<outname>] <filename> [<tags>]\n",argv[0]);
				exit(0);
				break;
		}
	}
	if(verbose)
		printf("Tag Organizer - Copyright (C) 1996 by Gran Gustafsson.\n\n");
	if(optind>=argc)
		Error(ERR_FATAL,"Missing filename");
	dl_InitHD(&tlhd);
	if(verbose)
		printf("Loading tagfile: %s\n",argv[optind]);
	LoadTagList(argv[optind],&tlhd);
	if(com)
	{
		if(optind+1>=argc)
			Error(ERR_FATAL,"No tags to process");
		for(n=optind+1; n<argc; n++)
		{
			switch(com)
			{
				case 'a':
					if(verbose)
						printf("Adding: %s - ",argv[n]);
					if(AddTag(argv[n],&tlhd))
					{
						chg=TRUE;
						if(verbose)
							printf("new\n");
					}
					else
					{
						if(verbose)
							printf("existed\n");
					}
					break;
				case 'd':
					if(verbose)
						printf("Removing: %s - ",argv[n]);
					if(RemTag(argv[n],&tlhd))
					{
						chg=TRUE;
						if(verbose)
							printf("ok\n");
					}
					else
					{
						if(verbose)
							printf("missing\n");
					}
					break;
				case 'f':
					printf("Searching for: %s - ",argv[n]);
					if(FindTag(argv[n],&tlhd))
						printf("found\n");
					else
						printf("missing\n");
					break;
			}
		}
	}
	if(list)
		ListTags(&tlhd);
	if(gen)
	{
		if(!out)
		{
			AppendSuffix(argv[optind],buf,".h");
			out=buf;
		}
		if(verbose)
			printf("Generating headerfile: %s\n",out);
		Generate(out,pref,start,nr,inc,&tlhd);
	}
	if(chg)
	{
		if(verbose)
			printf("Saving changes to: %s\n",argv[optind]);
		SaveTagList(argv[optind],&tlhd);
	}
	return(0);
}
